<!DOCTYPE html>
<html lang="en">
<head>
    <meta charset="UTF-8">
    <title>Title</title>
</head>
<body>
    <div>
        <a href="/">Home</a>
        <a href="/uploadproject">Upload</a>
    </div>
    <table>
        <tr>
            <th>Project</th>
            <th>Spider</th>
        </tr>
       {% for spider in spiders %}
        <tr>
            <td>
                {{spider.project.name}}
            </td>
            <td>
                <a href="/projects/{{spider.project.name}}/spiders/{{spider.name}}">{{spider.name}}</a>
            </td>
        </tr>
       {% end %}
    </table>
</body>
</html>