<!DOCTYPE html>
<html lang="en">
<head>
    <meta charset="UTF-8">
    <title>Log</title>
</head>
<body>
    <pre style="word-wrap: break-word; white-space: pre-wrap;">
        {{log}}
    </pre>
</body>
</html>